/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <bblnc.h>
#include <window.h>
#include <mainwindow.h>
#include <notifywindow.h>

///////////////////////////////////////////////////////////////////////////////
// Global Variables
///////////////////////////////////////////////////////////////////////////////
HINSTANCE   g_hInst;
CAppConfig* g_pAppConfigUser;
BOOL        g_fIsWinNT;
HKEY        g_hKeyAS;
INT			g_nCurState;
BOOL        g_fFirstStart;
BOOL		g_fUpdateThrdReady;
double		g_dAccStateLast;
BOOL		g_fShowErrors;

///////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////
INT APIENTRY _tWinMain(HINSTANCE hInst_, HINSTANCE, LPTSTR lpCmdLine_, INT nCmdShow_)
{
	MSG             msg;
	OSVERSIONINFOEX ovi;
	BOOL            fOsVerInfoEx;
	HWND            hWnd;

	cfdprintf(_T("_tWinMain"));

	g_hInst            = hInst_;
	g_nCurState        = BBLNC_CURRENT_STATE_DATA_NA;
	g_fFirstStart      = TRUE;
	g_fUpdateThrdReady = FALSE;
	g_dAccStateLast    = -4000000;  // Some value.

	// XXX Check, if there is already running instance, popup its config dialog window.
	hWnd = ::FindWindow(NULL, _T("Botik::Balance Configuration..."));
	if(hWnd)
	{
		::ShowWindow(hWnd, SW_SHOW);
		::ShowWindow(hWnd, SW_RESTORE);
		::SetForegroundWindow(hWnd);
		return(0);
	};

	// Get to know what platform we are running on (9x,NT).
	// Try calling GetVersionEx using the OSVERSIONINFOEX structure.
	// If that fails, try using the OSVERSIONINFO structure.
	ZeroMemory(&ovi, sizeof(OSVERSIONINFOEX));
	ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if(!(fOsVerInfoEx = GetVersionEx((LPOSVERSIONINFO)&ovi)))
	{
		ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
		if(!GetVersionEx((LPOSVERSIONINFO)&ovi))
			return(FALSE);
	};

	// Get platform flag.
	if(ovi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		g_fIsWinNT = TRUE;
	else
		g_fIsWinNT = FALSE;

	// For debug purposes...
//	g_fIsWinNT = FALSE;

	g_pAppConfigUser = new CAppConfig();
	if(!g_pAppConfigUser)
	{
		cfdprintf(_T("_tWinMain: Create g_pAppConfigGlobal object failed"));
		return(-1);
	};

	if(!g_pAppConfigUser->Open(HKEY_CURRENT_USER, _T("SOFTWARE\\Synapsis\\BotikBalance")))
	{
		cfdprintf(_T("_tWinMain: Call to g_pAppConfigUser->Open failed"));
		return(-1);
	};

	g_pAppConfigUser->SetAutoSave(TRUE);

	// Prelead configuration...
	g_pAppConfigUser->GetValue(BBLNC_FIRST_START, (LPDWORD)&g_fFirstStart, TRUE);

	g_pAppConfigUser->GetValue(BBLNC_ACCOUNTNAME, NULL, _T(""));
	g_pAppConfigUser->GetValue(BBLNC_USER, NULL, _T(""));
	g_pAppConfigUser->GetValue(BBLNC_PASSWD, NULL, _T(""));
	g_pAppConfigUser->GetValue(BBLNC_AUTOUPDATE, NULL, (DWORD)TRUE);
	g_pAppConfigUser->GetValue(BBLNC_SHOWINFOWINDOW, NULL, (DWORD)0);
	g_pAppConfigUser->GetValue(BBLNC_CRITICALACCOUNTSTATE, NULL, (DWORD)0);
	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTIMEOUT, NULL, (DWORD)10);
	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTRANSP, NULL, (DWORD)100);
	g_pAppConfigUser->GetValue(BBLNC_UPDATETIMEOUT, NULL, (DWORD)10);
	g_pAppConfigUser->GetValue(BBLNC_SHOWERRORS, (LPDWORD)&g_fShowErrors, (DWORD)TRUE);

	g_pAppConfigUser->GetValue(BBLNC_PROXYSERVER, NULL, (LPTSTR)NULL);

	RegCreateKeyEx(HKEY_CURRENT_USER, BBLNC_REG_AUTOSTART_PATH, 0, NULL, 
				   REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, 
				   NULL, &g_hKeyAS, NULL);

	if(!CNotifyWindow::CreateObject())
	{
		cfdprintf(_T("_tWinMain: CNotifyWindow::CreateObject failed"));
		return(-1);
	};

	NotifyWindow->Create(NULL);

	if(!CMainWindow::CreateObject())
	{
		cfdprintf(_T("_tWinMain: CMainWindow::CreateObject failed"));
		return(-1);
	};

	MainWindow->Create(NULL);

	MainWindow->Startup();

	if(g_fFirstStart)
	{
		::MessageBox(NULL, _T("!   ") BBLNC_MSGBOX_CAPTION _T("  .\n")
						   _T("    ,    .\n\n")
						   _T("   ! (C) . .\n\n")
						   _T("Have fun!!!"), 
					 BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONEXCLAMATION);

		MainWindow->Show(SW_SHOW);
	};

	// Main message loop.
	while(GetMessage(&msg, NULL, 0, 0)) 
	{
		if(!IsDialogMessage(MainWindow->GetHWnd(), &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		};
	};

	cfdprintf(_T("!_tWinMain"));

	return((INT)msg.wParam);
};



