/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <bblnc.h>
#include <window.h>
#include <notifywindow.h>

CNotifyWindow* NotifyWindow = NULL;

CNotifyWindow::CNotifyWindow(void) : CWindow()
{
	cfdprintf(_T("CNotifyWindow::CNotifyWindow"));
	
	m_lWndWidth    = 0;
	m_lWndHeight   = 0;
	m_fShow        = FALSE;
	m_szText       = NULL;

	m_crWndBgColor         = RGB(255, 255, 255);
	m_crWndFgColor         = RGB(0, 0, 0);
	m_crWndFrmNormLColor   = RGB(50, 122, 233); // RGB(132, 183, 233);
	m_crWndFrmNormDColor   = RGB(25, 40, 160);  // RGB(47, 74, 120);
	m_crWndFrmWarnLColor   = RGB(233, 122, 50);
	m_crWndFrmWarnDColor   = RGB(160, 40, 25);
	m_crWndFrmOutlineColor = m_crWndBgColor;

	cfdprintf(_T("!CNotifyWindow::CNotifyWindow"));
};

CNotifyWindow::~CNotifyWindow(void)
{
	cfdprintf(_T("CNotifyWindow::~CNotifyWindow"));
	cfdprintf(_T("!CNotifyWindow::~CNotifyWindow"));
};

BOOL CNotifyWindow::CreateObject()
{
	assert(NotifyWindow == NULL);
	NotifyWindow = new CNotifyWindow();
	return(TRUE);
};

BOOL CNotifyWindow::Create(HWND hWndPrn_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CNotifyWindow::Create"));

	assert(m_hWnd == NULL);

	m_hWnd = ::CreateDialog(g_hInst, MAKEINTRESOURCE(IDD_NOTIFYWND_DLG), NULL, (DLGPROC)_WndProc);

	if(!m_hWnd)
	{
		cfdprintf(_T("CNotifyWindow::Create: CreateDialog failed"));
		fOk = FALSE;
	};

	cfdprintf(_T("!CNotifyWindow::Create"));

	return(fOk);
};

BOOL CNotifyWindow::SetWindowPosition()
{
	RECT rcScr;

	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rcScr, 0);

	this->MoveTo(rcScr.right-m_lWndWidth-1, rcScr.bottom-m_lWndHeight, m_lWndWidth, m_lWndHeight);

	return(TRUE);
};

VOID CNotifyWindow::DrawFrame(HDC hdc_, CONST LPRECT prc_, COLORREF cr1_, COLORREF cr2_)
{
	HPEN		hPen, hPenOld;

	hPen = ::CreatePen(PS_SOLID, 2, cr1_);
	hPenOld = (HPEN)::SelectObject(hdc_, hPen);

	::MoveToEx(hdc_, prc_->left+1, prc_->bottom, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+1, prc_->top);
	::LineTo(hdc_, prc_->right, prc_->top+1);

	::MoveToEx(hdc_, prc_->left, prc_->bottom-3, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->right-3, prc_->bottom-3);
	::LineTo(hdc_, prc_->right-3, prc_->top);
	
	hPen = ::CreatePen(PS_SOLID, 2, cr2_);
	::DeleteObject(::SelectObject(hdc_, hPen));

	::MoveToEx(hdc_, prc_->left+3, prc_->bottom-4, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+3, prc_->top+2);
	::LineTo(hdc_, prc_->right-4, prc_->top+3);

	::MoveToEx(hdc_, prc_->left+1, prc_->bottom-1, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->right-1, prc_->bottom-1);
	::LineTo(hdc_, prc_->right-1, prc_->top+2);

	::SetPixel(hdc_, prc_->left+1, prc_->bottom-1, cr2_);
	::SetPixel(hdc_, prc_->left+2, prc_->bottom-3, cr2_);
	::SetPixel(hdc_, prc_->right-1, prc_->top+1, cr2_);
	::SetPixel(hdc_, prc_->right-3, prc_->top+2, cr2_);

	hPen = ::CreatePen(PS_SOLID, 1, m_crWndFrmOutlineColor);
	::DeleteObject(::SelectObject(hdc_, hPen));

	::MoveToEx(hdc_, prc_->left+4, prc_->bottom-5, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+4, prc_->top+4);
	::LineTo(hdc_, prc_->right-5, prc_->top+4);
	::LineTo(hdc_, prc_->right-5, prc_->bottom-5);
	::LineTo(hdc_, prc_->left+4, prc_->bottom-5);

	::DeleteObject(::SelectObject(hdc_, hPenOld));
};

VOID CNotifyWindow::PopUp()
{
	DWORD        dwTimeOut;

	if(m_fShow) return;

	m_fShow = TRUE;
	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTIMEOUT, &dwTimeOut, 0);
	NotifyWindow->Show(SW_SHOW);

	if(dwTimeOut > 0)
		::SetTimer(m_hWnd, NOTIFYWND_TIMER_ID, dwTimeOut*1000, (TIMERPROC)NULL);
};

VOID CNotifyWindow::Hide()
{
	m_fShow = FALSE;
	::KillTimer(m_hWnd, NOTIFYWND_TIMER_ID);
	NotifyWindow->Show(SW_HIDE);
};

VOID CNotifyWindow::SetText(LPTSTR szText_)
{
	RECT rc;

	if(m_szText)
		delete[] m_szText;

	m_szText = new TCHAR[_tcslen(szText_)+1];
	_tcscpy(m_szText, szText_);

	::GetClientRect(m_hWnd, &rc);
	::InvalidateRect(m_hWnd, &rc, TRUE);
};

///////////////////////////////////////////////////////////////////////////////
// WM event handlers
///////////////////////////////////////////////////////////////////////////////

LRESULT CNotifyWindow::OnInitDialog(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnInitDialog"));

	m_hWnd       = hWnd_;
	m_lWndWidth  = 220;
	m_lWndHeight = 110;

	this->SetWindowPosition();

	cfdprintf(_T("!CNotifyWindow::OnInitDialog"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnPaint(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	PAINTSTRUCT ps;
	HDC         hdc;
	RECT        rc, rcCapt, rcTxt;
	HBRUSH      hBr;
	COLORREF	crCapt;
	LOGFONT		lf;
	HFONT       hFont;
	LONG		lBgLogoID;
	BITMAP		bm;
	HBITMAP		hBmp;
	HDC			hMemDc;
	UINT		nFlags;
	HBITMAP     hBmpOld;
	HFONT		hFontOld;

	cfdprintf(_T("CNotifyWindow::OnPaint"));

	hdc = ::BeginPaint(hWnd_, &ps);

	::GetClientRect(hWnd_, &rc);

	// Fill window background
	hBr = ::CreateSolidBrush(m_crWndBgColor);
	::FillRect(hdc, &rc, hBr);
	::DeleteObject(hBr);

	// Draw background image
	lBgLogoID = g_nCurState == BBLNC_CURRENT_STATE_OVER_VALUE ? IDB_BGLOGO_WARN : IDB_BGLOGO_NORM;
	hBmp = (HBITMAP)::LoadImage(g_hInst, MAKEINTRESOURCE(lBgLogoID), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR|LR_CREATEDIBSECTION);
	if(hBmp == NULL)
		return(FALSE);

	::GetObject(hBmp, sizeof(BITMAP), &bm);

	hMemDc = ::CreateCompatibleDC(hdc);
	hBmpOld = (HBITMAP)::SelectObject(hMemDc, hBmp);
	::SetMapMode(hMemDc, ::GetMapMode(hdc));
	::BitBlt(hdc, rc.right-bm.bmWidth-5, rc.bottom-bm.bmHeight-5, bm.bmWidth, bm.bmHeight, hMemDc, 0, 0, SRCCOPY);

	::DeleteObject(::SelectObject(hMemDc, hBmpOld));
	::DeleteObject(hBmp);
	::DeleteDC(hMemDc);

	// Draw caption
	if(g_nCurState != BBLNC_CURRENT_STATE_OVER_VALUE)
		crCapt = m_crWndFrmNormLColor;
	else
		crCapt = m_crWndFrmWarnLColor;

	rcCapt = rc;
	rcCapt.bottom = 20;
	hBr = ::CreateSolidBrush(crCapt);
	::FillRect(hdc, (LPRECT)&rcCapt, hBr);
	::DeleteObject(hBr);

	// Draw window frame
	if(g_nCurState != BBLNC_CURRENT_STATE_OVER_VALUE)
		this->DrawFrame(hdc, &rc, m_crWndFrmNormLColor, m_crWndFrmNormDColor);
	else
		this->DrawFrame(hdc, &rc, m_crWndFrmWarnLColor, m_crWndFrmWarnDColor);

	// Get default GUI font information.
	::GetObject(::GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &lf);
	lf.lfWeight = FW_BOLD;
	lf.lfHeight = 12;
//	lf.lfWidth  = 5;
	lf.lfQuality = PROOF_QUALITY;
	hFont = ::CreateFontIndirect(&lf);
	hFontOld = (HFONT)::SelectObject(hdc, hFont);

	::SetBkMode(hdc, TRANSPARENT);

	// Write out caption
	LPTSTR szTxt = _T("  ");
	rcTxt = rcCapt;
	rcTxt.left += 7;
	rcTxt.top += 6;
	rcTxt.right -= 5;
	::SetTextColor(hdc, m_crWndBgColor);
	::DrawText(hdc, szTxt, (INT)_tcslen(szTxt), &rcTxt, DT_CENTER);

	::DeleteObject(::SelectObject(hdc, hFontOld));

	// Write out caption
	if(m_szText)
	{
		szTxt = m_szText;
		nFlags = 0;
		lf.lfWeight = FW_NORMAL;
		rcTxt.top = rcCapt.bottom + 4;
	}
	else
	{
		szTxt = _T(" ...");
		nFlags = DT_CENTER;
		lf.lfWeight = FW_BOLD;
		rcTxt.top = rcCapt.bottom + 30;
	};

	rcTxt.bottom = rc.bottom - 6;
	
//	lf.lfHeight = 11;
//	lf.lfWidth  = 6;
	hFont = CreateFontIndirect(&lf);
	hFontOld = (HFONT)::SelectObject(hdc, hFont);

	::SetTextColor(hdc, m_crWndFgColor);
	::DrawText(hdc, szTxt, (INT)_tcslen(szTxt), &rcTxt, nFlags);

	::DeleteObject(::SelectObject(hdc, hFontOld));

	::EndPaint(hWnd_, &ps);

	cfdprintf(_T("!CNotifyWindow::OnPaint"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnDestroy(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnDestroy"));

	::KillTimer(hWnd_, NOTIFYWND_TIMER_ID);
	PostQuitMessage(0);

	cfdprintf(_T("!CNotifyWindow::OnDestroy"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnCommand(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnCommand"));

	LRESULT lRes = FALSE;
	INT wmId     = LOWORD(wParam_);
	INT wmEvent  = HIWORD(wParam_);

//	switch(wmId)
//	{
//		// Menu commands...
//	};

	cfdprintf(_T("!CNotifyWindow::OnCommand (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CNotifyWindow::OnNotify(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnNotify"));
	cfdprintf(_T("!CNotifyWindow::OnNotify"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnTimer(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	if(wParam_ == NOTIFYWND_TIMER_ID)
		this->Hide();

	return(TRUE);
};

///////////////////////////////////////////////////////////////////////////////
// Window procedures
///////////////////////////////////////////////////////////////////////////////

LRESULT CNotifyWindow::WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	LRESULT lRes = 0;

//	cfdprintf(_T("CNotifyWindow::WndProc"));

	switch(uMsg_)
	{
		case WM_INITDIALOG:
			lRes = OnInitDialog(hWnd_, wParam_, lParam_);
			break;

		case WM_PAINT:
			lRes = OnPaint(hWnd_, wParam_, lParam_);
			break;

		case WM_DESTROY:
			lRes = OnDestroy(hWnd_, wParam_, lParam_);
			break;

		case WM_COMMAND:
			lRes = OnCommand(hWnd_, wParam_, lParam_);
			break;

		case WM_NOTIFY:
			lRes = OnNotify(hWnd_, wParam_, lParam_);
			break;

		case WM_TIMER:
			lRes = OnTimer(hWnd_, wParam_, lParam_);
			break;

		case WM_LBUTTONUP:
			this->Hide();
			break;

		default:
			break;
	};

//	cfdprintf(_T("!CNotifyWindow::WndProc (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CNotifyWindow::_WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	assert(NotifyWindow != NULL);
	return(NotifyWindow->WndProc(hWnd_, uMsg_, wParam_, lParam_));
};

